const int   sensorLength = 4;
const int   LEDpin = 13;

boolean     runFlag = false;
boolean     readSensors = false;
long        currentTimeCount = 0;
unsigned long samplingTime = 200; // ms
unsigned long timeRef;

/*
ICON(일부만 공개)
    ...
  6  Barometer.png
    ...
  9  CarbonDioxide.png
    ...
  14  Current.png
    ...
  20  Force.png
    ...
  31  Motion.png
    ...
  32  pH.png
    ...
  40  RelativePressure.png
    ...
  45  Temperature.png
    ...
  52  Voltage.png

단위 계열(일부만 공개)
  100: none
  101: Time
  102: Temperature
  103: Pressure
  104: Voltage
  105: Length
  106: Force
    ...
  108: Acceleration
    ...
  114: Current
    ...
  120: Parts Per Notation(농도 표기)
  121: Percent(%)
    ...
*/

typedef struct {
  uint8_t   Id;
  char      Name[32];
  char      Unit[8];
  uint8_t   Icon;           // ICON
  uint8_t   Calibration;
  uint8_t   SignInversion;
  uint8_t   RangeCount;
  uint8_t   SensorGroup;    // 단위 계열
  float     RealLowValue;
  float     RealHighValue;
} typeSensor;
typeSensor  Sensor[sensorLength];
float       sensorValue[sensorLength];

String      usbBuffer = "";

//=======================================================================
void setup() {
  init_Sensor();
  pinMode(LEDpin, OUTPUT);
  Serial.begin(115200);
}

void loop() {
  while(Serial.available() > 0) {
    char ch = Serial.read();
    if (ch == '{') 
      usbBuffer = "";
    else if (ch == '}') {
      parsing();
      usbBuffer = "";
    } else
      usbBuffer += ch;
  }

  if (runFlag) {
    if ((micros() - timeRef) / 1000 >= samplingTime) readSensors = true;
  }

  if(readSensors) readCurrentValue();
}

//=======================================================================
// 센서에 맞추어 수정하십시오
//-----------------------------------------------------------------------
void init_Sensor() {
  Sensor[0].Id = 1;
  strlcpy(Sensor[0].Name, "Force", 31);
  strlcpy(Sensor[0].Unit, "N", 7);
  Sensor[0].Icon = 20;
  Sensor[0].Calibration = 1;
  Sensor[0].SignInversion = 1;
  Sensor[0].RangeCount = 1;
  Sensor[0].SensorGroup = 106;
  Sensor[0].RealLowValue = -80;
  Sensor[0].RealHighValue = 80;

  Sensor[1].Id = 2;
  strlcpy(Sensor[1].Name, "Temperature", 31);
  strlcpy(Sensor[1].Unit, "'C", 7);
  Sensor[1].Icon = 45;
  Sensor[1].Calibration = 0;
  Sensor[1].SignInversion = 0;
  Sensor[1].RangeCount = 1;
  Sensor[1].SensorGroup = 102;
  Sensor[1].RealLowValue = -20;
  Sensor[1].RealHighValue = 120;
  
  Sensor[2].Id = 0;
  strlcpy(Sensor[2].Name, "none", 31);
  strlcpy(Sensor[2].Unit, "n", 7);
  Sensor[2].Icon = 0;
  Sensor[2].Calibration = 0;
  Sensor[2].SignInversion = 0;
  Sensor[2].RangeCount = 1;
  Sensor[2].SensorGroup = 100;
  Sensor[2].RealLowValue = 0;
  Sensor[2].RealHighValue = 100;
  
  Sensor[3].Id = 0;
  strlcpy(Sensor[3].Name, "none", 31);
  strlcpy(Sensor[3].Unit, "n", 7);
  Sensor[3].Icon = 0;
  Sensor[3].Calibration = 0;
  Sensor[3].SignInversion = 0;
  Sensor[3].RangeCount = 1;
  Sensor[3].SensorGroup = 100;
  Sensor[3].RealLowValue = 0;
  Sensor[3].RealHighValue = 100;
}

void readCurrentValue() {
  timeRef = micros();
  readSensors = false;
  {
    sensorValue[0]=analogRead(0);
  }
  {
    sensorValue[1]=analogRead(1);
  }

  Serial.print("{ 26,");
  for(int i = 0; i < sensorLength; i++) {
    if(i > 0) Serial.print(",");
    Serial.print(sensorValue[i]);
  }
  Serial.println(" }");
}

//=======================================================================
// 로깅프로그램 명령어 처리
//-----------------------------------------------------------------------
void parsing() {
  uint8_t i = 0;
  int incomingByte1 = 0;
  int incomingByte2 = 0;
  int incomingByte3 = 0;
  float retSampling = 0;
  float newTime = 0;
  
  incomingByte1 = getPasingValue(usbBuffer, ',', 0).toInt();
  switch(incomingByte1) {
    case 1:
      incomingByte2 = getPasingValue(usbBuffer, ',', 1).toInt();
      switch(incomingByte2) {
        case 1: // 인터페이스 확인
          Serial.println("{ 11,STEAMp.com }");
          Serial.flush();
          break;
        case 2: // 연결할 수 있는 센서의 수
          Serial.print("{ 12,");
          Serial.print(sensorLength);
          Serial.println(" }");
          break;
      }
      break;
    case 2: // send sensor information
      incomingByte2 = getPasingValue(usbBuffer, ',', 1).toInt();
      incomingByte3 = getPasingValue(usbBuffer, ',', 2).toInt();
      switch(incomingByte2) {
        case 1:   // 센서 ID (범위: 1~15)
          Serial.print("{ 13,");
          for(i = 0; i < sensorLength; i++) {
            if(i > 0) Serial.print(",");
            Serial.print(Sensor[i].Id);
          }
          Serial.println(" }");
          break;
        case 5:   // 센서 명칭
          Serial.print("{ 24,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].Name);
          Serial.println(" }");
          break;
        case 6:   // 센서 단위
          Serial.print("{ 25,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].Unit);
          Serial.println(" }");
          break;
        case 10:  // 센서 단위 계열
          Serial.print("{ 30,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].SensorGroup);
          Serial.println(" }");
          break;
        case 11:  // 센서 ICON
          Serial.print("{ 70,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].Icon);
          Serial.println(" }");
          break;
        case 12:  // 교정 대상인지?
          Serial.print("{ 71,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].Calibration);
          Serial.println(" }");
          break;
        case 13:  // 측정 범위를 선택하는 센서인지?
          Serial.print("{ 72,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].RangeCount);
          Serial.println(" }");
          break;
        case 14:  // 측정 값의 증가하는 방향이 상대적인지?
          Serial.print("{ 73,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].SignInversion);
          Serial.println(" }");
          break;
        case 16:  // 측정 값 표시 범위
          Serial.print("{ 75,");
          Serial.print(incomingByte3);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].RealLowValue);
          Serial.print(",");
          Serial.print(Sensor[incomingByte3].RealHighValue);
          Serial.println(" }");
          break;
      }
      break;
    case 3:
      incomingByte2 = getPasingValue(usbBuffer, ',', 1).toInt();
      switch(incomingByte2) {
        case 1:
          newTime = getPasingValue(usbBuffer, ',', 2).toFloat();
          if(0.05<= newTime && newTime <= 30.0) samplingTime = (unsigned long)(newTime * 1000.0);
          Serial.print("{ 15,");
          retSampling = samplingTime/1000.0;
          Serial.print(retSampling, 3);
          Serial.println(" }"); 
          break;
        case 2:
          if (!runFlag) readSensors = true;
          break;
        case 3:
          if (!runFlag) {
            currentTimeCount = 0;
            digitalWrite(LEDpin, true);
            Serial.println("{ 17 }");
            runFlag = true;
          }
          break;
        case 4:
          runFlag = false;
          Serial.println("{ 18 }");
          digitalWrite(LEDpin, false);
          break;
      }
      break;
  }
}

//-----------------------------------------------------------------------
String getPasingValue(String data, char separator, int index) {
  int found = 0;
  int strIndex[] = {0, -1};
  int maxIndex = data.length()-1;

  for(int i=0; i<=maxIndex && found<=index; i++){
    if(data.charAt(i)==separator || i==maxIndex){
        found++;
        strIndex[0] = strIndex[1]+1;
        strIndex[1] = (i == maxIndex) ? i+1 : i;
    }
  }

  return found>index ? data.substring(strIndex[0], strIndex[1]) : "";
}
